//
//  ContentView.swift
//  UkiCalendar
//
//  Created by ukiLab on 2023/10/23.
//

import SwiftUI

struct ContentView: View {
  @State var currentDate: Date = Date()
  @EnvironmentObject var calendarDate: CalendarDate
  @State private var pageIndex = 0
  @State private var isPresented: Bool = false

  var body: some View {
    VStack {
      NavigationView {
        VStack {
          InfinitePageView(
            selection: $pageIndex,
            before: { $0 - 1 },
            after: { $0 + 1 },
            view: { index in
              let targetDate = Calendar.current.date(byAdding: .month, value: index, to: currentDate) ?? Date()
              MonthView(currentDate: targetDate)
                .id(index)
                // セルのタップイベント
                // 同じセルタップしても動作するようにUUIDを利用
                .onChange(of: calendarDate.uuid) {
                  if (index != pageIndex) {
                    return
                  }
                  print("page index: \(pageIndex)")
                  print("selectedDate: \(calendarDate.selectedDate.dateTimeString())")
                  isPresented.toggle()
                }
                // スワイプしたらヘッダの表示が当月に変わるよう
                .onChange(of: pageIndex, { oldValue, newValue in
                  if (index != pageIndex) {
                    return
                  }
                  calendarDate.lastSelectedYearMonth = targetDate.yearMonth()
                  calendarDate.showingDate = targetDate
                }
              )
            }
          )
        }
        .navigationBarTitle(calendarDate.showingDate.yearMonthString(), displayMode: .inline)
        .navigationBarItems(leading: Button(action: {
          print("menu")
        }) {
          Text("Menu")
        }, trailing:  HStack {
          Button(action: {
            print("today")
            pageIndex = 0
            calendarDate.selectedDate = Date()
          }) {
            Text("Today")
          }
        })
      }
      .sheet(isPresented: $isPresented) {
        EventFormView(date: calendarDate.selectedDate)
          .onAppear() {
            print("year month: \(calendarDate.selectedDate.yearMonth()), last: \(calendarDate.lastSelectedYearMonth)")
            if (calendarDate.selectedDate.yearMonth() < calendarDate.lastSelectedYearMonth) {
              print("last month")
              pageIndex -= 1
            } else if (calendarDate.selectedDate.yearMonth() > calendarDate.lastSelectedYearMonth) {
              print("next month")
              pageIndex += 1
            }
            calendarDate.lastSelectedYearMonth = calendarDate.selectedDate.yearMonth()
          }
      }
    }
  }
  private func correctedIndex(for index: Int) -> Int {
    return index
  }
}

#Preview {
  ContentView().environmentObject(CalendarDate())
}
