//
//  MonthView.swift
//  UkiCalendar
//
//  Created by ukiLab on 2023/10/29.
//

import SwiftUI

struct MonthView: View {
  private let calendar = Calendar(identifier: .gregorian)
  private let currentDate: Date
  private let year: Int
  private let month: Int
  private let offset: Int
  private let endOfMonth: Int

  init(currentDate: Date) {
    self.currentDate = currentDate
    print("target: \(self.currentDate.dateTimeString())")
    year = self.currentDate.year
    month = self.currentDate.month
    offset = self.currentDate.startOfMonth.weekdayOfDay
    endOfMonth = self.currentDate.endOfMonth.day
//    print("year: \(year), month: \(month), offset: \(offset), endOfMonth: \(endOfMonth)")
  }
  
  var body: some View {
    VStack {
      WeekLabels()
      Grid(horizontalSpacing: 0, verticalSpacing: 0) {
        ForEach((0...5), id: \.self) { row in
          GridRow {
            ForEach((0...6), id: \.self) { column in
              let date = dateOfIndex(idx: row * 7 + column)
              MonthCell(targetMonth: month, date: date)
                .border(.secondary)
            }
          }
        }
      }
    }
  }
  
  func dateOfIndex(idx: Int) -> Date {
    let day = idx - offset + 2
    if ( day < 0) {
      let lastMonthLastDay = self.currentDate.endOfLastMonth
      return calendar.date(from: DateComponents(year: lastMonthLastDay.year, month: lastMonthLastDay.month, day: lastMonthLastDay.day + day))!
    } else if (day > endOfMonth) {
      let nextMonthFirstDay = self.currentDate.startOfNextMonth
      return calendar.date(from: DateComponents(year: nextMonthFirstDay.year, month: nextMonthFirstDay.month, day: day - endOfMonth))!
    }
    return calendar.date(from: DateComponents(year: year, month: month, day: day))!
    
  }
}

#Preview {
  MonthView(currentDate: Date())
    .environmentObject(CalendarDate())
}
