//
//  DateExtensionTests.swift
//  UkiCalendarTests
//
//  Created by ukiLab on 2023/10/30.
//

import XCTest
@testable import UkiCalendar

final class DateExtensionTests: XCTestCase {

  override func setUpWithError() throws {
    // Put setup code here. This method is called before the invocation of each test method in the class.
  }

  override func tearDownWithError() throws {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
  }

  func testYear1() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 1, day: 1))!
    XCTAssertEqual(date.year, 2023)
  }

  func testYear2() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2022, month: 12, day: 31))!
    XCTAssertEqual(date.year, 2022)
  }

  func testYear3() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 1969, month: 12, day: 31))!
    XCTAssertEqual(date.year, 1969)
  }

  func testMonth1() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 1, day: 1))!
    XCTAssertEqual(date.month, 1)
  }

  func testMonth2() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2022, month: 12, day: 31))!
    XCTAssertEqual(date.month, 12)
  }

  func testMonth3() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2020, month: 2, day: 29))!
    XCTAssertEqual(date.month, 2)
  }

  func testDay1() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 1, day: 1))!
    XCTAssertEqual(date.day, 1)
  }

  func testDay2() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2022, month: 12, day: 31))!
    XCTAssertEqual(date.day, 31)
  }

  func testDay3() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2020, month: 2, day: 29))!
    XCTAssertEqual(date.day, 29)
  }

  func testYearMonth1() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 1969, month: 12, day: 31))!
    XCTAssertEqual(date.yearMonth(), 196912)
  }

  func testYearMonth2() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 1, day: 1))!
    XCTAssertEqual(date.yearMonth(), 202301)
  }

  func testYearMonthString1() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 1969, month: 12, day: 31))!
    XCTAssertEqual(date.yearMonthString(), "1969-12")
  }

  func testYearMonthString2() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 1, day: 1))!
    XCTAssertEqual(date.yearMonthString(), "2023-01")
  }


  func testStartOfMonth1() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 3, day: 1))!
    XCTAssertEqual(date.startOfMonth.dateString(), "2023-03-01")
  }

  func testStartOfMonth2() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 3, day: 5))!
    XCTAssertEqual(date.startOfMonth.dateString(), "2023-03-01")
  }

  func testStartOfMonth3() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 3, day: 31))!
    XCTAssertEqual(date.startOfMonth.dateString(), "2023-03-01")
  }

  func testStartOfNextMonth1() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2020, month: 2, day: 1))!
    XCTAssertEqual(date.startOfNextMonth.dateString(), "2020-03-01")
  }

  func testEndOfLastMonth1() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2020, month: 3, day: 1))!
    XCTAssertEqual(date.endOfLastMonth.dateString(), "2020-02-29")
  }

  func testEndOfLastMonth2() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 12, day: 8))!
    print("endOfLastMonth: \(date.endOfLastMonth.dateString())")
    XCTAssertEqual(date.endOfLastMonth.dateString(), "2023-11-30")
  }

  func testWeekdayOfDay1() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 3, day: 1))!
    XCTAssertEqual(date.weekdayOfDay, 4)
  }

  func testWeekdayOfDay2() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2023, month: 2, day: 28))!
    XCTAssertEqual(date.weekdayOfDay, 3)
  }

  func testWeekdayOfDay3() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2020, month: 2, day: 29))!
    XCTAssertEqual(date.weekdayOfDay, 7)
  }

  func testWeekdayOfDay4() throws {
    let calendar = Calendar(identifier: .gregorian)
    let date = calendar.date(from: DateComponents(year: 2020, month: 3, day: 31))!
    XCTAssertEqual(date.endOfLastMonth.weekdayOfDay, 7)
  }

  func testPerformanceExample() throws {
    // This is an example of a performance test case.
    self.measure {
      // Put the code you want to measure the time of here.
    }
  }

}
